@@ -185,17 +185,17 @@
    :coerce-to-lisp-float nil
    :abs #'(lambda (s) (simplify (mfuncall '$cabs s)))
    :great #'(lambda (a b) (declare (ignore a)) (eq t (meqp b 0)))
-   :add #'(lambda (a b) ($rectform (add a b)))
-   :div #'(lambda (a b) ($rectform (div a b)))
-   :rdiv #'(lambda (a b) ($rectform (div a b)))
+   :add #'(lambda (a b) (add a b))
+   :div #'(lambda (a b) (div a b))
+   :rdiv #'(lambda (a b) (div a b))
    :reciprocal #'(lambda (s) (div 1 s))
-   :mult #'(lambda (a b) ($rectform (mult a b)))
-   :sub #'(lambda (a b) ($rectform (sub a b)))
+   :mult #'(lambda (a b) (mult a b))
+   :sub #'(lambda (a b) (sub a b))
    :negate #'(lambda (a) (mult -1 a))
    :psqrt #'(lambda (s) (if (member (csign s) `($pos $pz $zero)) (take '(%sqrt) s) nil))
    :add-id #'(lambda () 0)
    :mult-id #'(lambda () 1)
-   :fzerop #'(lambda (s) (eq t (meqp s 0)))
+   :fzerop #'(lambda (s) (eq t (meqp (sratsimp s) 0)))
    :adjoint #'(lambda (s) (take '($conjugate) s))
    :mring-to-maxima #'(lambda (s) s)
    :maxima-to-mring #'(lambda (s) s)))
