@@ -268,6 +268,20 @@
      (push (or d '(0 . 1)) row)
      (go loop2)))
 
+(defmfun $invert_by_gausselim (k) 
+  (let ((*inv* t) *det* linind* top* mul* ($ratmx t) (ratmx $ratmx) $ratfac $sparse)
+    (cond ((atom k) ($nounify '$inverx) (list '(%inverx) k))
+	  (t (newvarmat1 (setq k (check k)))
+	     (setq k (invert1 (replist1 (mcx (cdr k)))))
+	     (setq k (cond ($detout `((mtimes)
+				      ((mexpt) ,(rdis (or *det* '(1 . 1))) -1)
+				      (($matrix) ,@(mxc (disreplist1 k)))))
+			   (t (cons '($matrix) (mxc (disreplist1 k))))))
+	     (cond ((and ratmx (not $detout))
+		    (fmapl1 #'(lambda (x) x) k))
+		   ((not ratmx) ($totaldisrep k))
+		   (t k))))))
+
 (defun diaginv (ax m)
   (setq ax (get-array-pointer ax))
   (cond ($detout (setq *det* 1)
@@ -284,6 +298,23 @@
 					(pquotient (car *det*) (car elm))) 1))
 		 (t (ratinvert elm))))))
 
+(defun invert1 (k) 
+  (prog (l r g i m n ei* ej* oneoff*) 
+     (setq l (length k) i 1) 
+     (cond ((= l (length (car k))) nil)
+	   (t(merror (intl:gettext "invert: matrix must be square; found ~M rows, ~M columns.") l (length (car k)))))
+     loop (cond ((null k) (go l1))) 
+     (setq r (car k)) 
+     (setq g (nconc g (list (nconc r (onen i l '(1 . 1) '(0 . 1)))))) 
+     (setq k (cdr k) i (1+ i)) 
+     (go loop) 
+     l1   (setq k g)
+     (mtoa '*mat* (setq m (length k)) (setq n (length (car k))) k)
+     (setq k nil)
+     (cond ((diagp '*mat* m) (diaginv '*mat* m)) (t (tfgeli0 '*mat* m n)))
+     (setq k (atomat '*mat* m n (1+ m)))
+     (return k)))
+
 (defun diagp (ax m)
   (declare (fixnum m))
   (prog ((i 0) (j 0))
