@@ -618,10 +618,10 @@
 	(t x)))
 
 (defmspec $string (form)
-  (setq form (strmeval (fexprcheck form)))
-  (setq form (if $grind (strgrind form) (mstring form)))
-  (setq st (reverse form))
-  (coerce form 'string))
+  (let (($lispdisp t))
+    (setq form (strmeval (fexprcheck form)))
+    (setq form (if $grind (strgrind form) (mstring form)))
+    (coerce form 'string)))
 
 (defmfun makstring (x)
   (setq x (mstring x))
