@@ -12,8 +12,6 @@
 
 (macsyma-module trutil)
 
-(transl-module trutil)
-
 ;;; takes a list, and returns a cons of an a-list of (gensym . exp)
 ;;; and the origonal list with gensyms substututed for non-atom elements
 ;;; of the list. This could be used to define subr-like makros.
@@ -35,7 +33,7 @@
 	   ;; push one with a priority of 1, which will be over-rided
 	   ;; by any user-specified settings.
 	   (if (eq $define_variable '$mode)
-	       (tr-format "~%Note: ~:M being given a default setting of ~:M"
+	       (tr-format (intl:gettext "note: variable ~:M being given a default assignment ~:M~%")
 			  var (if (atom val) val
 				  ;; strip off the quote
 				  (cadr val))))
@@ -63,13 +61,12 @@
        (do ((entry))
 	   ((null new-entries))
 	 (setq entry (pop new-entries))
-	 (or (member entry *new-autoload-entries* :test #'eq)
-	     (push-pre-transl-form
+	 (push-pre-transl-form
 	      `(putprop ',entry
 			;; this ensures that the autoload definition
 			;; will not get out of date.
 		(or (get ',old-entry 'autoload) t)
-		'autoload))))))
+		'autoload)))))
 
 (defun tr-nargs-check (form &optional (args-p nil) (nargs (length (cdr form))))
   ;; the maclisp args info format is NIL meaning no info,
@@ -79,11 +76,11 @@
 	     (min (or (car args-p) (cdr args-p)))
 	     (max (cdr args-p)))
 	 (cond ((and min (< nargs min))
-		(mformat *translation-msgs-files* "~%Error: Too few arguments supplied to ~:@M~%"
+		(mformat *translation-msgs-files* (intl:gettext "error: too few arguments supplied to ~:@M~%")
 			 (caar form))
 		(mgrind form *translation-msgs-files*))
 	       ((and max (> nargs max))
-		(tr-format  "~%Error: Too many arguments supplied to ~:@M~%" (caar form))
+		(tr-format  (intl:gettext "error: too many arguments supplied to ~:@M~%") (caar form))
 		(mgrind form *translation-msgs-files*)))))
   nargs) ;; return the number of arguments.
 
