@@ -11,8 +11,6 @@
 (in-package :maxima)
 (macsyma-module ufact)
 
-(declare-top (special modulus coef-type))
-
 (load-macsyma-macros ratmac rzmac)
 
 ;; Dense Polynomial Representation
@@ -148,7 +146,7 @@
 
 (defun ufact-strip-zeroes (l)
   (do ((l l (cdr l)))
-      ((null (czerop (car l))) l)))
+      ((null (pzerop (car l))) l)))
 
 (defun cpres1 (a b)
   (prog (res (v 0) a3) (declare (fixnum v))
